@@ -76,14 +76,14 @@ class KDNode(_core.KDNode):
 
         return _core.KDNode.count(self, other, r, attrs, info=info)
 
-    def fof(self, linkinglength, out=None):
+    def fof(self, linkinglength, out=None, method='splay'):
         """ Friend-of-Friend clustering with linking length.
 
             Returns: the label
         """
         if out is None:
             out = numpy.empty(self.size, dtype='intp')
-        return _core.KDNode.fof(self, linkinglength, out)
+        return _core.KDNode.fof(self, linkinglength, out, method)
 
     def integrate(self, min, max, attr=None, info={}):
         """ Calculate the total number of points between [min, max).
@@ -143,11 +143,12 @@ class KDTree(_core.KDTree):
             the size of the torus is given by boxsize.
         thresh : int
             minimal size of a leaf.
-
+        ind : array_like, or None
+            indices of rows in the input that are used in the tree.
     """
     __nodeclass__ = KDNode
-    def __init__(self, input, boxsize=None, thresh=10):
-        _core.KDTree.__init__(self, input, boxsize, thresh)
+    def __init__(self, input, boxsize=None, thresh=10, ind=None):
+        _core.KDTree.__init__(self, input, boxsize, thresh, ind)
 
     def __repr__(self):
         return ('KDTree(size=%d, thresh=%d, boxsize=%s, input=%s)' % 
@@ -195,7 +196,7 @@ def makeiter(feeder):
             yield item
             q.task_done()
 
-from numpy.testing import Tester
-test = Tester().test
-bench = Tester().bench
-
+force_kernels = {
+    'count': _core.KDForceCount(),
+    'plummer': _core.KDForcePlummer,
+}
